import imp.DIMACS_GraphImporter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;

import algorithms.graphColorer.heuristics.GreedyColorer;
import algorithms.graphColorer.reoptimizer.IncConsColoringSolver;
import algorithms.graphColorer.reoptimizer.IteratedGreedyColorer;
import dataStructures.graph.Edge;
import dataStructures.graph.ExtendedUndirectedGraph;
import dataStructures.graph.GraphColoring;

public class GraphColorer
{
	public static void main(String[] args) throws IOException
	{
		if ( args == null || args.length == 0 )
			throw new IllegalArgumentException("FILENAME and SOLVER expected");

		if ( args[0] == null )
			throw new IllegalArgumentException("filepath expected");

		if ( args[1] == null )
			throw new IllegalArgumentException("SOLVER parameter expected");

		final String fileName = args[0];
		final int algorithm = Integer.parseInt(args[1]);

		if ( algorithm < 1 || algorithm > 3 )
			throw new IllegalArgumentException("SOLVER parameter must be in the range of 1...3");

		final ExtendedUndirectedGraph<Integer, Edge<Integer>> graph = DIMACS_GraphImporter.getGraph(fileName);

		GraphColoring<Integer, Edge<Integer>> coloring;
		final GraphColoring<Integer, Edge<Integer>> greedyColoring = GreedyColorer.getBestGreedyColoring(graph);
		if ( algorithm == 1 )
		{
			coloring = greedyColoring;
		}
		else if ( algorithm == 2 )
		{
			coloring = new IteratedGreedyColorer<Integer, Edge<Integer>>(graph, 1000, greedyColoring).getColoring();
		}
		else
		{
			coloring = new IncConsColoringSolver<Integer, Edge<Integer>>(graph, greedyColoring).getColoring();
		}

		System.out.println("#colors: " + coloring.getNumberOfColors());
		HashMap<Integer, HashSet<Integer>> colorVertexMapping = coloring.getColorVertexMapping();
		for (int color : colorVertexMapping.keySet())
		{
			ArrayList<Integer> vertexSet = new ArrayList<Integer>(colorVertexMapping.get(color));
			Collections.sort(vertexSet);
			System.out.println("color: " + color + ", vertices: " + vertexSet);
		}

	}
}
