package algorithms.graphColorer.heuristics;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.PriorityQueue;
import java.util.Set;

import org.jgrapht.UndirectedGraph;

public class GreedyStrategy<V, E>
{

	public static final int[] COLOR_STRATEGY = { GreedyStrategy.LARGEST_FIRST_GREEDY_STRATEGY,
			GreedyStrategy.SMALLEST_FIRST_GREEDY_STRATEGY, GreedyStrategy.SIMPLE_GREEDY_STRATEGY,
			GreedyStrategy.REVERSE_ORDER_GREEDY_STRATEGY, GreedyStrategy.RANDOM_SEQUENCE_GREEDY_STRATEGY };

	public static final int LARGEST_FIRST_GREEDY_STRATEGY = 2;

	public static final int RANDOM_SEQUENCE_GREEDY_STRATEGY = 4;

	public static final int REVERSE_ORDER_GREEDY_STRATEGY = 5;

	public static final int SIMPLE_GREEDY_STRATEGY = 1;

	public static final int SMALLEST_FIRST_GREEDY_STRATEGY = 3;

	private final int colorStrategy;

	public GreedyStrategy()
	{
		final double ran = Math.random();
		if ( ran <= 22.5 )
		{
			this.colorStrategy = GreedyStrategy.LARGEST_FIRST_GREEDY_STRATEGY;
		}
		else if ( ran <= 45 )
		{
			this.colorStrategy = GreedyStrategy.SMALLEST_FIRST_GREEDY_STRATEGY;
		}
		else if ( ran <= 67.5 )
		{
			this.colorStrategy = GreedyStrategy.SIMPLE_GREEDY_STRATEGY;
		}
		else if ( ran <= 90 )
		{
			this.colorStrategy = GreedyStrategy.REVERSE_ORDER_GREEDY_STRATEGY;
		}
		else
		{
			this.colorStrategy = GreedyStrategy.RANDOM_SEQUENCE_GREEDY_STRATEGY;
		}
	}

	public GreedyStrategy(int colorStrategy)
	{
		this.colorStrategy = colorStrategy;
	}

	public int getColorStrategy()
	{
		return this.colorStrategy;
	}

	public ArrayList<V> getSortedVertexSet(UndirectedGraph<V, E> g)
	{
		class NodeDegreeComp implements Comparator<V>
		{
			private final UndirectedGraph<V, E> g;

			public NodeDegreeComp(UndirectedGraph<V, E> graph)
			{
				this.g = graph;
			}

			public int compare(V o1, V o2)
			{
				return this.g.degreeOf(o2) - this.g.degreeOf(o1);
			}
		}
		class Vertex implements Comparable<Vertex>
		{
			private final int degree;

			private final V v;

			public Vertex(V v, int degee)
			{
				this.v = v;
				this.degree = degee;
			}

			public int compareTo(Vertex o)
			{
				return o.degree - this.degree;
			}
		}
		final Set<V> vertexSet = g.vertexSet();
		final ArrayList<V> out = new ArrayList<V>(vertexSet.size());
		final PriorityQueue<Vertex> prio = new PriorityQueue<Vertex>(vertexSet.size());
		for (final V v : vertexSet)
		{
			prio.add(new Vertex(v, g.degreeOf(v)));
		}

		while (prio.peek() != null)
		{
			out.add(prio.poll().v);
		}
		return out;
	}
}
