package algorithms.graphColorer.reoptimizer;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Vector;

import org.jgrapht.UndirectedGraph;

import algorithms.graphColorer.heuristics.GreedyStrategy;
import dataStructures.graph.GraphColoring;

public class ExtendedGreedy<V, E> extends GreedyStrategy<V, E>
{
	private final GraphColoring<V, E> coloring;

	private final int permutationStrategy;

	public ExtendedGreedy(GraphColoring<V, E> coloring, int colorStrategy, int permutationStrategy)
	{
		super(colorStrategy);
		this.permutationStrategy = permutationStrategy;
		this.coloring = coloring;
	}

	@Override
	public ArrayList<V> getSortedVertexSet(UndirectedGraph<V, E> g)
	{
		final ArrayList<V> vSet = new ArrayList<V>();
		final HashMap<Integer, HashSet<V>> map = this.coloring.getColorVertexMapping();
		if ( this.permutationStrategy == IteratedGreedyColorer.LARGEST_FIRST_PERMUTATION )
		{
			class LargestColorClass implements Comparator<HashSet<V>>
			{
				public int compare(HashSet<V> o1, HashSet<V> o2)
				{
					return o2.size() - o1.size();
				}
			}
			final Vector<HashSet<V>> sortedColorClasses = new Vector<HashSet<V>>();
			sortedColorClasses.addAll(map.values());
			Collections.sort(sortedColorClasses, new LargestColorClass());
			for (final HashSet<V> colorClass : sortedColorClasses)
			{
				vSet.addAll(colorClass);
			}
		}
		else if ( this.permutationStrategy == IteratedGreedyColorer.REVERSE_PERMUTATION )
		{
			final ArrayList<Integer> allColors = new ArrayList<Integer>(map.keySet());
			Collections.sort(allColors, Collections.reverseOrder());
			for (final int color : allColors)
			{
				vSet.addAll(map.get(color));
			}
		}
		else
		{
			final ArrayList<Integer> allColors = new ArrayList<Integer>(map.keySet());
			class RandomComp implements Comparator<Integer>
			{
				public int compare(Integer o1, Integer o2)
				{
					final double random = Math.random();
					return random < .4 ? 1 : random < .6 ? 0 : -1;
				}
			}
			Collections.sort(allColors, new RandomComp());
			for (final int color : allColors)
			{
				vSet.addAll(map.get(color));
			}
		}
		return vSet;
	}
}
