package algorithms.graphColorer.reoptimizer;

import java.util.Random;

import algorithms.graphColorer.heuristics.GreedyColorer;
import algorithms.graphColorer.heuristics.GreedyStrategy;
import dataStructures.graph.ExtendedUndirectedGraph;
import dataStructures.graph.GraphColoring;

public class IteratedGreedyColorer<V, E> extends ColoringReoptimizer<V, E>
{

	public static final int LARGEST_FIRST_PERMUTATION = 0;

	public static int[] permutationStrategies = { IteratedGreedyColorer.LARGEST_FIRST_PERMUTATION,
			IteratedGreedyColorer.REVERSE_PERMUTATION, IteratedGreedyColorer.RANDOM_PERMUTATION };

	public static final int RANDOM_PERMUTATION = 2;

	public static final int REVERSE_PERMUTATION = 1;

	public static <T> void createRandomPermutation(T[] arr)
	{
		final Random rand = new Random();
		int last = arr.length - 1; // last is maximal index of elements not in the tail

		while (last > 0)
		{
			final T tmp = arr[last];
			final int index = rand.nextInt(last + 1);
			arr[last] = arr[index];
			arr[index] = tmp;
			last -= 1;
		}

	}

	public int iterations;

	final int maxIteration;

	public IteratedGreedyColorer(ExtendedUndirectedGraph<V, E> g, int maxIteration, GraphColoring<V, E> cloring)
	{
		super(g, cloring);
		this.maxIteration = maxIteration;
	}

	public GraphColoring<V, E> getColoring()
	{
		for (int i = 0; i < this.maxIteration; i++)
		{

			int permutationStrategy;
			final double ran = Math.random();
			if ( ran <= 0.38 )
			{
				permutationStrategy = IteratedGreedyColorer.LARGEST_FIRST_PERMUTATION;
			}
			else if ( ran <= 0.78 )
			{
				permutationStrategy = IteratedGreedyColorer.REVERSE_PERMUTATION;
			}
			else
			{
				permutationStrategy = IteratedGreedyColorer.RANDOM_PERMUTATION;
			}

			for (final int strategy : GreedyStrategy.COLOR_STRATEGY)
			{
				this.iterations++;
				final GreedyStrategy<V, E> stra = new ExtendedGreedy<V, E>(this.coloring, strategy, permutationStrategy);
				final GreedyColorer<V, E> colorer = new GreedyColorer<V, E>(this.graph, stra);
				final GraphColoring<V, E> tmpColor = colorer.getColoring();

				if ( tmpColor.getNumberOfColors() < this.coloring.getNumberOfColors() )
				{
					i = 0;
				}
				this.coloring = tmpColor;
			}
		}

		return this.coloring;
	}

	public int getNumberOfIterations()
	{
		return this.iterations;
	}

}
