package dataStructures.graph;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.TreeMap;
import java.util.Vector;

import org.jgrapht.EdgeFactory;

public class ExtendedSimpleGraphAjazenzList<V> implements ExtendedUndirectedGraph<V, Edge<V>>
{
	private final HashMap<V, HashSet<V>> adjazenzlist;

	public ExtendedSimpleGraphAjazenzList()
	{
		this.adjazenzlist = new HashMap<V, HashSet<V>>();
	}

	public ExtendedSimpleGraphAjazenzList(int n)
	{
		this.adjazenzlist = new HashMap<V, HashSet<V>>(n);
	}

	public Edge<V> addEdge(V sourceVertex, V targetVertex)
	{
		this.adjazenzlist.get(sourceVertex).add(targetVertex);
		this.adjazenzlist.get(targetVertex).add(sourceVertex);
		return new Edge<V>(sourceVertex, targetVertex);
	}

	public boolean addEdge(V sourceVertex, V targetVertex, Edge<V> e)
	{
		if ( e.getSource().equals(sourceVertex) && e.getTarget().equals(targetVertex) )
		{
			this.addEdge(sourceVertex, targetVertex);
			return true;
		}
		return false;
	}

	public boolean addVertex(V v)
	{
		this.adjazenzlist.put(v, new HashSet<V>());
		return true;
	}

	public boolean containsEdge(Edge<V> e)
	{
		return this.containsEdge(e.getSource(), e.getTarget());
	}

	public boolean containsEdge(V sourceVertex, V targetVertex)
	{
		if ( sourceVertex.equals(targetVertex) )
			return false;
		return this.adjazenzlist.get(sourceVertex).contains(targetVertex);
	}

	public boolean containsVertex(V v)
	{
		return this.adjazenzlist.containsKey(v);
	}

	public int degreeOf(V vertex)
	{
		return this.adjazenzlist.get(vertex).size();
	}

	public Set<Edge<V>> edgeSet()
	{
		final Set<Edge<V>> edgeSet = new HashSet<Edge<V>>();
		for (final V v : this.vertexSet())
		{
			edgeSet.addAll(this.edgesOf(v));
		}
		return edgeSet;
	}

	public Set<Edge<V>> edgesOf(V vertex)
	{
		final Set<Edge<V>> edgeSet = new HashSet<Edge<V>>();
		for (final V v : this.adjazenzlist.get(vertex))
		{
			edgeSet.add(new Edge<V>(v, vertex));
		}
		return edgeSet;
	}

	public Set<Edge<V>> getAllEdges(V sourceVertex, V targetVertex)
	{
		// TODO Auto-generated method stub
		return null;
	}

	public Edge<V> getEdge(V sourceVertex, V targetVertex)
	{
		if ( this.adjazenzlist.get(sourceVertex).contains(targetVertex) )
			return new Edge<V>(sourceVertex, targetVertex);
		return null;
	}

	public EdgeFactory<V, Edge<V>> getEdgeFactory()
	{
		// TODO Auto-generated method stub
		return null;
	}

	public V getEdgeSource(Edge<V> e)
	{
		return e.getSource();
	}

	public V getEdgeTarget(Edge<V> e)
	{
		return e.getTarget();
	}

	public double getEdgeWeight(Edge<V> e)
	{
		// TODO Auto-generated method stub
		return 0;
	}

	public Collection<V> getNeighbors(V v)
	{
		return this.adjazenzlist.get(v);
	}

	public TreeMap<Integer, Vector<V>> getNeighborsPerColor(V vertex, GraphColoring<V, Edge<V>> coloring)
	{
		final TreeMap<Integer, Vector<V>> out = new TreeMap<Integer, Vector<V>>();
		for (final V n : this.getNeighbors(vertex))
		{
			if ( coloring.isAlreadyColored(n) )
			{
				Vector<V> vec = out.get(coloring.getColor(n));
				if ( vec == null )
				{
					vec = new Vector<V>();
					out.put(coloring.getColor(n), vec);
				}
				vec.add(n);
			}
		}
		return out;
	}

	public boolean removeAllEdges(Collection<? extends Edge<V>> edges)
	{
		for (final Edge<V> e : edges)
		{
			this.removeEdge(e);
		}
		return true;
	}

	public Set<Edge<V>> removeAllEdges(V sourceVertex, V targetVertex)
	{
		if ( this.removeEdge(sourceVertex, targetVertex) != null )
		{
			final Set<Edge<V>> e = new HashSet<Edge<V>>();
			e.add(new Edge<V>(sourceVertex, targetVertex));
			return e;
		}
		return null;
	}

	public boolean removeAllVertices(Collection<? extends V> vertices)
	{
		for (final V v : vertices)
		{
			this.removeVertex(v);
		}
		return true;
	}

	public boolean removeEdge(Edge<V> e)
	{
		return this.removeEdge(e.getSource(), e.getTarget()) != null;
	}

	public Edge<V> removeEdge(V sourceVertex, V targetVertex)
	{
		if ( this.adjazenzlist.get(sourceVertex).contains(targetVertex) )
		{
			this.adjazenzlist.get(sourceVertex).remove(targetVertex);
			this.adjazenzlist.get(targetVertex).remove(sourceVertex);
			return new Edge<V>(sourceVertex, targetVertex);
		}
		return null;
	}

	public boolean removeVertex(V v)
	{
		if ( this.adjazenzlist.containsKey(v) )
		{
			final HashSet<V> n = this.adjazenzlist.get(v);
			for (final V ver : n)
			{
				this.adjazenzlist.get(ver).remove(v);
			}
			this.adjazenzlist.remove(v);
			return true;
		}
		return false;
	}

	public Set<V> vertexSet()
	{
		return this.adjazenzlist.keySet();
	}
}
