package dataStructures.utils;

import java.util.ArrayList;
import java.util.Set;

public final class ArrayS
{

	public static int max(Integer[] arr)
	{
		int max = Integer.MIN_VALUE;
		for (final int i : arr)
		{
			if ( i > max )
			{
				max = i;
			}
		}
		return max;
	}

	public static double mean(double[] greedyK)
	{
		double mean = 0;
		for (final double element : greedyK)
		{
			mean += element;
		}
		return mean / greedyK.length;
	}

	public static float mean(float[] greedyTimeA)
	{
		float mean = 0;
		for (final float element : greedyTimeA)
		{
			mean += element;
		}
		return mean / greedyTimeA.length;
	}

	public static double mean(int[] greedyK)
	{
		double mean = 0;
		for (final int element : greedyK)
		{
			mean += element;
		}
		return mean / greedyK.length;
	}

	/**
	 * Returns the index of the median of the three indexed integers.
	 */
	private static int med3(int x[], int[] p, int a, int b, int c)
	{
		return x[p[a]] < x[p[b]] ? (x[p[b]] < x[p[c]] ? b : x[p[a]] < x[p[c]] ? c : a) : x[p[b]] > x[p[c]] ? b
				: x[p[a]] > x[p[c]] ? c : a;
	}

	public static ArrayList<Integer> randomPermutation(Set<Integer> keySet)
	{
		final ArrayList<Integer> dataRecords = new ArrayList<Integer>(keySet);
		for (int step = 1; step < dataRecords.size(); step++)
		{
			for (int i = 0; i + step < dataRecords.size(); i++)
			{
				if ( Math.random() <= 0.5f )
				{
					final Integer tmp = dataRecords.get(i);
					dataRecords.set(i, dataRecords.get(i + step));
					dataRecords.set(i + step, tmp);
				}
			}
		}
		return dataRecords;
	}

	public static int[] sort(int[] arr)
	{
		final int[] permutationIndices = new int[arr.length];
		for (int i = 0; i < permutationIndices.length; i++)
		{
			permutationIndices[i] = i;
		}

		final int off = 0;
		final int len = arr.length;
		ArrayS.sort1(arr, permutationIndices, off, len);
		return permutationIndices;
	}

	public static void sort1(int[] arr, int[] p, int off, int len)
	{
		// Insertion sort on smallest arrays
		if ( len < 7 )
		{
			for (int i = off; i < len + off; i++)
			{
				for (int j = i; j > off && arr[p[j - 1]] > arr[p[j]]; j--)
				{
					ArrayS.swap(p, j, j - 1);
				}
			}
			return;
		}

		// Choose a partition element, v
		int m = off + (len >> 1); // Small arrays, middle element
		if ( len > 7 )
		{
			int l = off;
			int n = off + len - 1;
			if ( len > 40 )
			{ // Big arrays, pseudomedian of 9
				final int s = len / 8;
				l = ArrayS.med3(arr, p, l, l + s, l + 2 * s);
				m = ArrayS.med3(arr, p, m - s, m, m + s);
				n = ArrayS.med3(arr, p, n - 2 * s, n - s, n);
			}
			m = ArrayS.med3(arr, p, l, m, n); // Mid-size, med of 3
		}
		final int v = arr[p[m]];

		// Establish Invariant: v* (<v)* (>v)* v*
		int a = off, b = a, c = off + len - 1, d = c;
		while (true)
		{
			while (b <= c && arr[p[b]] <= v)
			{
				if ( arr[p[b]] == v )
				{
					ArrayS.swap(p, a++, b);
				}
				b++;
			}
			while (c >= b && arr[p[c]] >= v)
			{
				if ( arr[p[c]] == v )
				{
					ArrayS.swap(p, c, d--);
				}
				c--;
			}
			if ( b > c )
			{
				break;
			}
			ArrayS.swap(p, b++, c--);
		}

		// Swap partition elements back to middle
		int s;
		final int n = off + len;
		s = Math.min(a - off, b - a);
		ArrayS.vecswap(p, off, b - s, s);
		s = Math.min(d - c, n - d - 1);
		ArrayS.vecswap(p, b, n - s, s);

		// Recursively sort non-partition-elements
		if ( (s = b - a) > 1 )
		{
			ArrayS.sort1(arr, p, off, s);
		}
		if ( (s = d - c) > 1 )
		{
			ArrayS.sort1(arr, p, n - s, s);
		}
	}

	public static float standardDeviation(float[] greedyTimeA)
	{
		final double mean = ArrayS.mean(greedyTimeA);
		double stb = 0;
		for (final double element : greedyTimeA)
		{
			stb += Math.pow(element - mean, 2);
		}
		stb /= greedyTimeA.length - 1;
		return (float) Math.sqrt(stb);
	}

	public static double standardDeviation(int[] greedyK)
	{
		final double mean = ArrayS.mean(greedyK);
		double stb = 0;
		for (final int element : greedyK)
		{
			stb += Math.pow(element - mean, 2);
		}
		stb /= greedyK.length - 1;
		stb = Math.sqrt(stb);
		return stb;
	}

	/**
	 * Swaps x[a] with x[b].
	 */
	private static void swap(int[] p, int a, int b)
	{
		final int t = p[a];
		p[a] = p[b];
		p[b] = t;
	}

	/**
	 * Swaps x[a .. (a+n-1)] with x[b .. (b+n-1)].
	 */
	private static void vecswap(int p[], int a, int b, int n)
	{
		for (int i = 0; i < n; i++, a++, b++)
		{
			ArrayS.swap(p, a, b);
		}
	}
}
